#include "acdb.h"
void getXData(void * handle)
{
	HC_Open_Segment_By_Key(m_pHView->GetModelKey());
	{
		// In this example we inquire about the metadata for the entity with handle 74
		// the segment xdata will contain numerous subsegments. 
		// Each subsegment represents the metadata for one entity.
		// If an entity does not have metadata associated with it, there will not be a corresponding subsegment under xdata
		// The subsegments are named "0x" + entity_handle
		HC_Open_Segment(H_FORMAT_TEXT("xdata/0x%08x", handle));

			// Each subsegment contains ONE user data.
			// This user data tells us WHICH TYPE of metadata will be present for this unit.
			// In this code data_indices_count represents HOW MANY different kind of metadata are in this segmnet
			// available_xdata_types is an array which tells your WHAT those metadata are
			// Metadata types are stored by the enum given to them by Autodesk.
			// For example if available_xdata_type contains "1000" that means we have a metadata of type kDxfXdAsciiString
			long data_indices_count;
			POINTER_SIZED_INT available_xdata_types[128];
			data_indices_count = HC_Show_User_Data_Indices(available_xdata_types, 128); 
			int current_index = 0;
			while (current_index < data_indices_count)
			{
				// Each entity subsegment has additional subsegments which group together metadata of the same type
				// These subsegments are called restype xxxx, where xxxx is the type of metadata
				HC_Open_Segment(H_FORMAT_TEXT("restype %d", available_xdata_types[current_index]));
				{
					// now we check
					long num_of_metadata;
					POINTER_SIZED_INT entity_metadata[128];
					num_of_metadata = HC_Show_User_Data_Indices(entity_metadata, 128);

					// possible metadata format
					ACHAR * metadata_string = new ACHAR[1024];
					double metadata_double;
					double metadata_point[3];
					short metadata_short;
					long metadata_long;
					char metadata_binary_buffer[1024];
#if defined(WIN64) || defined(_M_X64) || defined(_WIN64)
					__int64 metadata_name;
#else
					long metadata_name;
#endif
					for (int i = 0; i < num_of_metadata; ++i)
					{
						switch (available_xdata_types[current_index])
						{
							// type - real
							case AcDb::kDxfXdReal:
							case AcDb::kDxfXdDist:
							case AcDb::kDxfXdScale:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)&metadata_double, sizeof(double));
								// the metadata is in metadata_double -- do something with it
							}
							break;

							// type - rpoint[3]
							case AcDb::kDxfXdXCoord:
							case AcDb::kDxfXdWorldXCoord:
							case AcDb::kDxfXdWorldXDisp:
							case AcDb::kDxfXdWorldXDir:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)metadata_point, sizeof(double) * 3);
								// the metadata is in metadata_double -- do something with it
							}
							break;

							// type - rint
							case AcDb::kDxfXdInteger16:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)&metadata_short, sizeof(short));
								// the metadata is in metadata_short -- do something with it
							}
							break;

							// type - ACHAR *
							case AcDb::kDxfXdAsciiString:
							case AcDb::kDxfRegAppName:
							case AcDb::kDxfXdControlString:
							case AcDb::kDxfXdLayerName:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)metadata_string, (long)(sizeof(ACHAR) * 1024));
								// the metadata is in metadata_string -- do something with it
							}
							break;

							// type - long rlname[2]
							case AcDb::kDxfXdHandle: 
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)&metadata_name, sizeof(metadata_name));
								// the metadata is in metadata_name -- do something with it
							}
							break;

							// type - long rlong
							case AcDb::kDxfXdInteger32:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)&metadata_long, sizeof(metadata_long));
								// the metadata is in metadata_long -- do something with it
							}
							break;

							case AcDb::kDxfXdBinaryChunk:
							{
								HC_Show_One_User_Data(entity_metadata[i], (void *)metadata_binary_buffer, sizeof(metadata_binary_buffer));
								// the metadata is in metadata_binary_buffer -- do something with it
							}
							break;

							default:
							break;
						} // end of switch
					} // end of loop
				}
				HC_Close_Segment();
				++current_index;
			}
		HC_Close_Segment();
	}
	HC_Close_Segment();
}